"""
inteltime - Cycle and Checkpoint Calculator front-end for Slack
"""

import json
import re
import pytz
from flask import Flask, Blueprint, Response, request

from . import Cycle, Checkpoint

bp = Blueprint('inteltime', __name__)                #pylint: disable=invalid-name

@bp.route("/cycle", methods=['GET', 'POST'])
def slack_checkpoint():
    """Compute cycle time"""
    form = request.form if request.method == 'POST' else request.args
    text = form.get('text', '')
    if ',' in text:
        target, zone = text.split(',')
    else:
        target, zone = text, None
    try:
        cycle = Cycle(target, timezone=zone)
    except pytz.exceptions.UnknownTimeZoneError as err:
        return "Unknown timezone: {}".format(err)

    text = "Cycle start: {}\nCycle end: {}".format(cycle, cycle+1)
    return Response(json.dumps({"text": text, "response_type": "in_channel"}),
                    content_type="application/json")


@bp.route("/checkpoint", methods=['GET', 'POST'])
def slack_cycle():
    """Comput checkpoint time, or checkpoints on date"""
    form = request.form if request.method == 'POST' else request.args
    text = form.get('text', '')
    if ',' in text:
        target, zone = text.split(',')
    else:
        target, zone = text, None
    match = re.match("on (.*)", target, re.IGNORECASE)
    try:
        if match:
            checkpoint = Checkpoint(match.group(0), timezone=zone)
            text = "\n".join(["Checkpoint {}: {}{}".format(
                chkp.number(), chkp, " (new cycle)" if chkp.cycle_start() else '')
                              for chkp in checkpoint.on_day()])
        else:
            checkpoint = Checkpoint(target, timezone=zone)
            text = "Next checkpoint at {} (in {})".format(
                checkpoint+1, checkpoint.until_next())
    except pytz.exceptions.UnknownTimeZoneError as err:
        text = "Error: Unknown timezone: {}".format(err)
    return Response(json.dumps({"text": text, "response_type": "in_channel"}),
                    content_type="application/json")


app = Flask(__name__)                            #pylint: disable=invalid-name
app.register_blueprint(bp)

def main():
    """Standalone flask mode for debugging, not designed for production"""
    config = {
        'url-prefix': '/inteltime',
        'ssl-context': ['/etc/ssl/private/local.pem'],
        'debug': True
    }

    url_prefix = config.get('url-prefix')
    if url_prefix:
        app.register_blueprint(bp, url_prefix=url_prefix)

    ssl_context = config.get('ssl-context')
    if ssl_context:
        ssl_context = tuple(ssl_context)

    app.run(
        host=config.get('bind-host', '0.0.0.0'),
        port=config.get('bind-port', 8443),
        debug=config.get('debug', False),
        ssl_context=ssl_context
    )


if __name__ == '__main__':
    main()



"""
Simple test jig (not yet automated) for intel-times library
"""
import argparse
from . import Cycle, Checkpoint

def main():
    """Class test/example code"""
    parser = argparse.ArgumentParser("checkpoint and cycle time test")
    parser.add_argument("time", type=str, nargs='?', help="optional time (default to now")
    parser.add_argument("--zone", "-z", type=str, help="optional timezone (default to local")
    args = parser.parse_args()

    cycle = Cycle(target=args.time, timezone=args.zone)
    print("Cycle start:", cycle)
    print("Cycle end:", cycle + 1)
    print("Cycle datetime objects:", cycle.start, cycle.end)
    print("Cycle as timestamp:", int(cycle))

    print("Cycle incremental add")
    cycle += 1
    print("New cycle:", cycle)
    print("New datetime objects:", cycle.start, cycle.end)
    cycle = cycle + 5
    print("Moved forward 5 cycles:", cycle)
    print("New datetime objects:", cycle.start, cycle.end)
    print()

    checkpoint = Checkpoint(target=args.time, timezone=args.zone)
    print("Checkpoint start:", checkpoint)
    print("Checkpoint end:", checkpoint + 1)
    print("Checkpoints since start of cycle:", checkpoint.number())
    print("Time since last:", checkpoint.since_last())
    print("Time until next:", checkpoint.until_next())

    checkpoint += 1
    print("Next checkpoint:", checkpoint)
    print("Checkpoints since start of cycle:", checkpoint.number())

    for day in ["04/18/2016", "yesterday", "today", "tomorrow", "April 25, 2016"]:
        print("Checkpoints {}:".format(day))
        for checkpoint in Checkpoint(day, timezone=args.zone).on_day():
            print("Checkpoint {}: {}{}".format(
                checkpoint.number(),
                checkpoint,
                ' (new cycle)' if checkpoint.cycle_start() else ''))

if __name__ == '__main__':
    main()
